@@ -143,9 +143,9 @@ QT_BEGIN_NAMESPACE
     local default Bluetooth adapter.
 */
 QBluetoothServiceDiscoveryAgent::QBluetoothServiceDiscoveryAgent(QObject *parent)
-: QObject(parent), d_ptr(new QBluetoothServiceDiscoveryAgentPrivate(QBluetoothAddress()))
+  : QObject(parent),
+    d_ptr(new QBluetoothServiceDiscoveryAgentPrivate(this, QBluetoothAddress()))
 {
-     d_ptr->q_ptr = this;
 }
 
 /*!
@@ -161,9 +161,9 @@ QBluetoothServiceDiscoveryAgent::QBluetoothServiceDiscoveryAgent(QObject *parent
     \sa error()
 */
 QBluetoothServiceDiscoveryAgent::QBluetoothServiceDiscoveryAgent(const QBluetoothAddress &deviceAdapter, QObject *parent)
-: QObject(parent), d_ptr(new QBluetoothServiceDiscoveryAgentPrivate(deviceAdapter))
+  : QObject(parent),
+    d_ptr(new QBluetoothServiceDiscoveryAgentPrivate(this, deviceAdapter))
 {
-    d_ptr->q_ptr = this;
     if (!deviceAdapter.isNull()) {
         const QList<QBluetoothHostInfo> localDevices = QBluetoothLocalDevice::allDevices();
         foreach (const QBluetoothHostInfo &hostInfo, localDevices) {
@@ -211,7 +211,9 @@ QList<QBluetoothServiceInfo> QBluetoothServiceDiscoveryAgent::discoveredServices
 }
 /*!
     Sets the UUID filter to \a uuids.  Only services matching the UUIDs in \a uuids will be
-    returned.
+    returned. The matching applies to the service's
+    \l {QBluetoothServiceInfo::ServiceId}{ServiceId} and \l {QBluetoothServiceInfo::ServiceClassIds} {ServiceClassIds}
+    attributes.
 
     An empty UUID list is equivalent to a list containing only QBluetoothUuid::PublicBrowseGroup.
 
@@ -228,6 +230,9 @@ void QBluetoothServiceDiscoveryAgent::setUuidFilter(const QList<QBluetoothUuid>
     This is an overloaded member function, provided for convenience.
 
     Sets the UUID filter to a list containing the single element \a uuid.
+    The matching applies to the service's \l {QBluetoothServiceInfo::ServiceId}{ServiceId}
+    and \l {QBluetoothServiceInfo::ServiceClassIds} {ServiceClassIds}
+    attributes.
 
     \sa uuidFilter()
 */
@@ -257,8 +262,9 @@ QList<QBluetoothUuid> QBluetoothServiceDiscoveryAgent::uuidFilter() const
     address can only be set while there is no service discovery in progress; otherwise
     this function returns false.
 
-    On some platforms such as Blackberry the service discovery might lead to pairing requests.
+    On some platforms the service discovery might lead to pairing requests.
     Therefore it is not recommended to do service discoveries on all devices.
+    This function can be used to restrict the service discovery to a particular device.
 
     \sa remoteAddress()
 */
@@ -289,7 +295,7 @@ QBluetoothAddress QBluetoothServiceDiscoveryAgent::remoteAddress() const
 /*!
     Starts service discovery. \a mode specifies the type of service discovery to perform.
 
-    On BlackBerry devices, device discovery may lead to pairing requests.
+    On some platforms, device discovery may lead to pairing requests.
 
     \sa DiscoveryMode
 */
