@@ -37,6 +37,7 @@
 #include "osx/osxbtsdpinquiry_p.h"
 #include "qbluetoothhostinfo.h"
 #include "osx/osxbtutility_p.h"
+#include "osx/osxbluetooth_p.h"
 #include "osx/uistrings_p.h"
 
 #include <QtCore/qloggingcategory.h>
@@ -47,8 +48,6 @@
 #include <QtCore/qlist.h>
 
 #include <Foundation/Foundation.h>
-// Only after Foundation.h
-#include "osx/corebluetoothwrapper_p.h"
 
 QT_BEGIN_NAMESPACE
 
